@@ -8,7 +8,7 @@
 #include "trace.h"
 
 #include "buffer.h"
-#include "global.h"
+#include "runtime.h"
 #include "git2/trace.h"
 
 #ifdef GIT_TRACE
@@ -17,10 +17,10 @@ struct git_trace_data git_trace__data = {0};
 
 #endif
 
-int git_trace_set(git_trace_level_t level, git_trace_callback callback)
+int git_trace_set(git_trace_level_t level, git_trace_cb callback)
 {
 #ifdef GIT_TRACE
-	assert(level == 0 || callback != NULL);
+	GIT_ASSERT_ARG(level == 0 || callback != NULL);
 
 	git_trace__data.level = level;
 	git_trace__data.callback = callback;
