@@ -53,7 +53,9 @@ static wchar_t* win32_walkpath(wchar_t *path, wchar_t *buf, size_t buflen)
 {
 	wchar_t term, *base = path;
 
-	assert(path && buf && buflen);
+	GIT_ASSERT_ARG_WITH_RETVAL(path, NULL);
+	GIT_ASSERT_ARG_WITH_RETVAL(buf, NULL);
+	GIT_ASSERT_ARG_WITH_RETVAL(buflen, NULL);
 
 	term = (*path == L'"') ? *path++ : L';';
 
@@ -109,7 +111,7 @@ static int win32_find_git_in_registry(
 	HKEY hKey;
 	int error = GIT_ENOTFOUND;
 
-	assert(buf);
+	GIT_ASSERT_ARG(buf);
 
 	if (!RegOpenKeyExW(hive, key, 0, KEY_READ, &hKey)) {
 		DWORD dwType, cbData;
