@@ -238,7 +238,7 @@ if ($is_gcc) {
 
 # there are no atomic primitives for the Sun Pro compiler in libgit2, so even if pthreads is available
 # and perl has been built with threads support, libgit2 cannot use threads under said compiler
-if ($Config{usethreads} && !$is_sunpro) {
+if (!$is_sunpro) {
 	if (check_lib(lib => 'pthread')) {
 		$def .= ' -DGIT_THREADS';
 		$lib .= ' -lpthread';
@@ -246,15 +246,14 @@ if ($Config{usethreads} && !$is_sunpro) {
 		print "Threads support enabled\n";
 	} else {
 		if ($is_windows) {
+			print "Threads support enabled\n";
 			$def .= ' -DGIT_THREADS';
 		} else {
 			print "Threads support disabled (pthreads not found)\n";
 		}
 	}
-} elsif ($is_sunpro) {
-	print "Thread support disabled (SunPro compiler detected)\n"
 } else {
-	print "Thread support disabled (perl wasn't built with thread support)\n"
+	print "Thread support disabled (SunPro compiler detected)\n"
 }
 
 my @deps = glob 'deps/libgit2/deps/{http-parser,zlib,pcre}/*.c';
